# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : user_service.py 
# @Software: Visual Studio Code

from logging import getLogger
import random
from typing import Any
from telethon.events import NewMessage, ButtonCallback
from sqlalchemy.exc import IntegrityError

from component.handle_context import HandleContext
from config import Config
from dao import transaction
from dao.model import Payment, User
from dao.payment_dao import PaymentDao
from dao.user_dao import UserDao

from nzooherd_pyzen.classes import Singleton

from component.pornherd_bot import mybot
from template.referral import ReferralSuccessTemplate, ReferralTemplate
from template.start import StartTemplate
from template.tip import TipTemplate
from utills.referral_code import generate_referral_code 

class UserService(metaclass=Singleton):

    logger = getLogger()

    def __init__(self) -> None:
        self.handle_context = HandleContext()
        self.bot = mybot

    def init_user_context(self, update: Any):
        if isinstance(update, NewMessage):
            self.handle_context.set_user_id(update.chat.id)
        if isinstance(update, ButtonCallback):
            self.handle_context.set_user_id(update._raw.user_id)
    
    def get_handle_context(self):
        return self.handle_context

    async def referral_link(self) -> ReferralTemplate:
        """
        返回引荐链接
        """
        user_id = self.handle_context.get_user_id()
        user = await UserDao.get_by_user_id(user_id)
        link = f"https://t.me/{Config.TELEGRAM_BOT_NAME}?start={user.referral_code}"
        return ReferralTemplate(link=link)


    async def random_tip(self) -> TipTemplate:
        """
        Return one tip randomly
        """
        all_tips = TipTemplate.tips
        return TipTemplate(random.choice(all_tips))

    @transaction
    async def new_user(self, update: NewMessage) -> StartTemplate:
        chat = update.chat
        chat_id = str(chat.id)
        referral_code = generate_referral_code()
        user = User(user_id=chat_id, user_ref=str(chat.ref), user_name=chat.username, lang_code="en", 
                    referral_code=referral_code)

        refarraled_code = ""
        if update.text != "/start":
            try:
                refarraled_code = update.text.split(" ")[1]
            except IndexError:
                self.logger.warn(f"Invalid referral code: {update.text}")
        try:
            if refarraled_code:
                refarraled_user = await UserDao.get_by_referral_code(refarraled_code)
                if not refarraled_user:
                    refarraled_code = None
            
            if refarraled_code:
                user.referraled_code = refarraled_code
                user_payment = await PaymentDao.modify_amount(refarraled_user.user_id, 1)

            await UserDao.insert(user)
            await PaymentDao.insert(Payment(user_id=chat_id))

            if refarraled_code:
                try:
                    referral_success_template = ReferralSuccessTemplate(referral_code, user_payment.amount)
                    await self.bot.send_message(refarraled_user.user_ref, **referral_success_template.template())
                except Exception as e:
                    self.logger.warn(f"Send referral success message error: {e}")
        except IntegrityError:
            self.logger.warn(f"User: {chat_id} already exists")
        finally:
            return StartTemplate()
